import { useMemo } from "react";
import { DmnBuiltInDataType } from "@kie-tools/boxed-expression-component/dist/api";
import { DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH } from "./DmnRunnerOutputsTable";
export function useDmnRunnerOutputs(jsonSchemaBridge, results) {
    return useMemo(() => {
        var _a, _b, _c;
        if (jsonSchemaBridge === undefined || results === undefined) {
            return { outputs: [] };
        }
        const outputTypeMap = Object.entries((_c = (_b = (_a = jsonSchemaBridge.schema.definitions) === null || _a === void 0 ? void 0 : _a.OutputSet) === null || _b === void 0 ? void 0 : _b.properties) !== null && _c !== void 0 ? _c : []).reduce((outputTypeMap, [name, properties]) => {
            var _a;
            if (properties["x-dmn-type"]) {
                const dataType = jsonSchemaBridge.getFieldDataType(properties).dataType;
                outputTypeMap.set(name, {
                    type: properties.type,
                    dataType,
                    name,
                    joinedName: name,
                });
            }
            else {
                const path = (_a = properties.$ref) === null || _a === void 0 ? void 0 : _a.split("/").slice(1);
                const data = path === null || path === void 0 ? void 0 : path.reduce((acc, property) => acc[property], jsonSchemaBridge.schema);
                const dataType = jsonSchemaBridge.getFieldDataType(data).dataType;
                if (data.properties) {
                    const insideProperties = deepGenerateOutputTypesMapFields(outputTypeMap, data.properties, jsonSchemaBridge);
                    outputTypeMap.set(name, {
                        type: data.type,
                        insideProperties,
                        dataType,
                        name,
                        joinedName: name,
                    });
                }
                else {
                    outputTypeMap.set(name, {
                        type: data.type,
                        dataType,
                        name,
                        joinedName: name,
                    });
                }
            }
            return outputTypeMap;
        }, new Map());
        const decisionResults = results === null || results === void 0 ? void 0 : results.filter((result) => result !== undefined);
        const outputMap = decisionResults.reduce((outputFieldsMap, decisionResult) => {
            if (decisionResult) {
                decisionResult.forEach(({ decisionName }) => {
                    var _a, _b;
                    const outputField = outputTypeMap.get(decisionName);
                    if (outputField && isOutputWithInsideProperties(outputField)) {
                        outputFieldsMap.set(decisionName, {
                            name: decisionName,
                            joinedName: decisionName,
                            dataType: (_a = outputField.dataType) !== null && _a !== void 0 ? _a : DmnBuiltInDataType.Undefined,
                            insideProperties: outputField.insideProperties,
                            width: outputField.insideProperties.reduce((outputFieldsMap, column) => { var _a; return outputFieldsMap + ((_a = column === null || column === void 0 ? void 0 : column.width) !== null && _a !== void 0 ? _a : DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH); }, 0),
                        });
                    }
                    else {
                        outputFieldsMap.set(decisionName, {
                            name: decisionName,
                            joinedName: decisionName,
                            dataType: (_b = outputField === null || outputField === void 0 ? void 0 : outputField.dataType) !== null && _b !== void 0 ? _b : DmnBuiltInDataType.Undefined,
                            width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                        });
                    }
                });
            }
            return outputFieldsMap;
        }, new Map());
        return {
            outputs: [...outputMap.values()],
            outputTypeMap,
        };
    }, [jsonSchemaBridge, results]);
}
function deepGenerateOutputTypesMapFields(outputTypeMap, properties, jsonSchemaBridge, parentName) {
    return Object.entries(properties).map(([name, property]) => {
        var _a;
        if (property["x-dmn-type"]) {
            const dataType = jsonSchemaBridge.getFieldDataType(property).dataType;
            outputTypeMap.set(name, {
                type: property.type,
                dataType,
                name,
                joinedName: parentName ? `${parentName}-${name}` : name,
            });
            return {
                name,
                type: property.type,
                width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                dataType,
                joinedName: parentName ? `${parentName}-${name}` : name,
            };
        }
        const path = (_a = property.$ref) === null || _a === void 0 ? void 0 : _a.split("/").slice(1);
        const field = path === null || path === void 0 ? void 0 : path.reduce((acc, property) => acc[property], jsonSchemaBridge.schema);
        const dataType = jsonSchemaBridge.getFieldDataType(field).dataType;
        if (field.properties) {
            const insideProperties = deepGenerateOutputTypesMapFields(outputTypeMap, field.properties, jsonSchemaBridge, name);
            outputTypeMap.set(name, {
                type: field.type,
                insideProperties,
                dataType,
                name,
                joinedName: name,
            });
            return {
                name,
                type: field.type,
                dataType,
                width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                insideProperties,
                joinedName: name,
            };
        }
        else {
            outputTypeMap.set(name, {
                type: field.type,
                dataType,
                name,
                joinedName: name,
            });
        }
        return {
            name,
            type: field.type,
            dataType,
            width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
        };
    });
}
export function isOutputWithInsideProperties(toBeDetermined) {
    return toBeDetermined.insideProperties !== undefined;
}
//# sourceMappingURL=DmnRunnerOutputs.js.map