import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { ErrorBoundary } from "@kie-tools/form/dist/ErrorBoundary";
import { EmptyState, EmptyStateBody, EmptyStateIcon } from "@patternfly/react-core/dist/js/components/EmptyState";
import { Text, TextContent } from "@patternfly/react-core/dist/js/components/Text";
import { CubeIcon } from "@patternfly/react-icons/dist/js/icons/cube-icon";
import { ExclamationIcon } from "@patternfly/react-icons/dist/js/icons/exclamation-icon";
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
import nextId from "react-id-generator";
import { useDmnRunnerOutputs as useDmnRunnerOutputs } from "./DmnRunnerOutputs";
import { BeeTableHeaderVisibility, DmnBuiltInDataType, generateUuid, } from "@kie-tools/boxed-expression-component/dist/api";
import { getColumnsAtLastLevel } from "@kie-tools/boxed-expression-component/dist/table/BeeTable";
import { StandaloneBeeTable } from "@kie-tools/boxed-expression-component/dist/table/BeeTable/StandaloneBeeTable";
import "@kie-tools/boxed-expression-component/dist/@types/react-table";
import { ResizerStopBehavior } from "@kie-tools/boxed-expression-component/dist/resizing/ResizingWidthsContext";
import "./DmnRunnerOutputsTable.css";
export function DmnRunnerOutputsTable({ i18n, jsonSchemaBridge, results, scrollableParentRef }) {
    const outputUid = useMemo(() => nextId(), []);
    const outputErrorBoundaryRef = useRef(null);
    const [outputError, setOutputError] = useState(false);
    const { outputs, outputTypeMap } = useDmnRunnerOutputs(jsonSchemaBridge, results);
    useEffect(() => {
        var _a;
        (_a = outputErrorBoundaryRef.current) === null || _a === void 0 ? void 0 : _a.reset();
    }, [outputs]);
    const rows = useMemo(() => (results !== null && results !== void 0 ? results : []).map((result) => ({
        outputEntries: (result !== null && result !== void 0 ? result : []).map(({ result }) => result),
    })), [results]);
    return (_jsx(_Fragment, { children: outputError ? (outputError) : outputs.length > 0 ? (_jsx(ErrorBoundary, Object.assign({ ref: outputErrorBoundaryRef, setHasError: setOutputError, error: _jsx(OutputError, {}, void 0) }, { children: _jsx(OutputsBeeTable, { scrollableParentRef: scrollableParentRef, i18n: i18n, outputs: outputs, outputTypeMap: outputTypeMap, rows: rows, id: outputUid }, void 0) }), void 0)) : (_jsxs(EmptyState, { children: [_jsx(EmptyStateIcon, { icon: CubeIcon }, void 0), _jsx(TextContent, { children: _jsx(Text, Object.assign({ component: "h2" }, { children: "No Decision results yet..." }), void 0) }, void 0), _jsx(EmptyStateBody, { children: _jsx(TextContent, { children: "Add input and decision nodes, provide values to the inputs at the left and see the Decisions results here." }, void 0) }, void 0)] }, void 0)) }, void 0));
}
function OutputError() {
    return (_jsx("div", { children: _jsxs(EmptyState, { children: [_jsx(EmptyStateIcon, { icon: ExclamationIcon }, void 0), _jsx(TextContent, { children: _jsx(Text, Object.assign({ component: "h2" }, { children: "Error" }), void 0) }, void 0), _jsx(EmptyStateBody, { children: _jsx("p", { children: "An error has happened while trying to show your Decision results" }, void 0) }, void 0)] }, void 0) }, void 0));
}
export const DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH = 150;
const EMPTY_SYMBOL = "";
function OutputsBeeTable({ id, i18n, outputs, outputTypeMap, rows, scrollableParentRef }) {
    const beeTableOperationConfig = useMemo(() => [
        {
            group: i18n.rows,
            items: [],
        },
    ], [i18n]);
    const uuid = useMemo(() => {
        return generateUuid();
    }, []);
    const deepFlattenObjectColumn = useCallback((myObject, parentKey) => {
        return Object.entries(myObject).flatMap(([myObjectKey, value]) => {
            if (value !== null && typeof value === "object") {
                const myKey = parentKey ? `${parentKey}-${myObjectKey}` : myObjectKey;
                return deepFlattenObjectColumn(value, myKey);
            }
            const label = parentKey ? `${parentKey}-${myObjectKey}` : myObjectKey;
            const myObjectProperties = outputTypeMap === null || outputTypeMap === void 0 ? void 0 : outputTypeMap.get(myObjectKey);
            const dataType = myObjectProperties ? myObjectProperties.dataType : DmnBuiltInDataType.Undefined;
            return {
                originalId: label + generateUuid(),
                label,
                accessor: (`output-object-${label}` + generateUuid()),
                dataType,
                isRowIndexColumn: false,
                groupType: "dmn-runner-output",
                width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                minWidth: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
            };
        });
    }, [outputTypeMap]);
    const deepFlattenObjectRow = useCallback((myObject, parentKey) => {
        return Object.entries(myObject).flatMap(([myObjectKey, value]) => {
            if (value !== null && typeof value === "object") {
                return deepFlattenObjectRow(value, myObjectKey);
            }
            return value;
        });
    }, []);
    const beeTableColumns = useMemo(() => {
        var _a, _b;
        return ((_b = (_a = rows === null || rows === void 0 ? void 0 : rows[0]) === null || _a === void 0 ? void 0 : _a.outputEntries) !== null && _b !== void 0 ? _b : []).flatMap((outputEntry, outputIndex) => {
            var _a, _b, _c, _d, _e;
            const output = outputs === null || outputs === void 0 ? void 0 : outputs[outputIndex];
            if (typeof outputEntry === "string" ||
                typeof outputEntry === "number" ||
                typeof outputEntry === "boolean" ||
                outputEntry === null) {
                return [
                    {
                        originalId: `-parent-${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid(),
                        label: "",
                        accessor: (`output-parent-${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid()),
                        dataType: undefined,
                        isRowIndexColumn: false,
                        groupType: "dmn-runner-output",
                        minWidth: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                        columns: [
                            {
                                originalId: `${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid(),
                                label: (_a = output === null || output === void 0 ? void 0 : output.name) !== null && _a !== void 0 ? _a : "",
                                accessor: (`output-${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid()),
                                dataType: (_b = output === null || output === void 0 ? void 0 : output.dataType) !== null && _b !== void 0 ? _b : DmnBuiltInDataType.Undefined,
                                isRowIndexColumn: false,
                                groupType: "dmn-runner-output",
                                width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                                minWidth: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                            },
                        ],
                    },
                ];
            }
            if (Array.isArray(outputEntry)) {
                return [
                    {
                        originalId: `${output === null || output === void 0 ? void 0 : output.name}}-` + generateUuid(),
                        label: `${output === null || output === void 0 ? void 0 : output.name}`,
                        accessor: (`output-array-parent-${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid()),
                        dataType: (_c = output === null || output === void 0 ? void 0 : output.dataType) !== null && _c !== void 0 ? _c : DmnBuiltInDataType.Undefined,
                        isRowIndexColumn: false,
                        groupType: "dmn-runner-output",
                        columns: outputEntry.map((entry, entryIndex) => ({
                            originalId: `${entryIndex}-` + generateUuid(),
                            label: `[${entryIndex}]`,
                            accessor: (`output-array-${entryIndex}-` + generateUuid()),
                            dataType: undefined,
                            groupType: "dmn-runner-output",
                            isRowIndexColumn: false,
                            width: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                            minWidth: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                        })),
                    },
                ];
            }
            if (typeof outputEntry === "object") {
                return [
                    {
                        originalId: `${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid(),
                        label: (_d = output === null || output === void 0 ? void 0 : output.name) !== null && _d !== void 0 ? _d : "",
                        accessor: (`output-object-parent-${output === null || output === void 0 ? void 0 : output.name}-` + generateUuid()),
                        dataType: (_e = output === null || output === void 0 ? void 0 : output.dataType) !== null && _e !== void 0 ? _e : DmnBuiltInDataType.Undefined,
                        isRowIndexColumn: false,
                        groupType: "dmn-runner-output",
                        minWidth: DMN_RUNNER_OUTPUT_COLUMN_MIN_WIDTH,
                        columns: deepFlattenObjectColumn(outputEntry),
                    },
                ];
            }
            return [];
        });
    }, [deepFlattenObjectColumn, outputs, rows]);
    const beeTableRows = useMemo(() => {
        return rows.map((row, rowIndex) => {
            const rowArray = row.outputEntries.reduce((acc, entry) => {
                if (entry === undefined) {
                    return acc;
                }
                else if (entry === null) {
                    return [...acc, "null"];
                }
                else if (Array.isArray(entry)) {
                    return [...acc, ...entry.map((element) => JSON.stringify(element, null, 2).replace(/"([^"]+)":/g, "$1:"))];
                }
                else if (typeof entry === "object") {
                    return [
                        ...acc,
                        ...deepFlattenObjectRow(entry).map((element) => JSON.stringify(element, null, 2).replace(/"([^"]+)":/g, "$1:")),
                    ];
                }
                else {
                    return [...acc, JSON.stringify(entry)];
                }
            }, []);
            return getColumnsAtLastLevel(beeTableColumns).reduce((tableRow, column, columnIndex) => {
                tableRow[column.accessor] = rowArray[columnIndex] || EMPTY_SYMBOL;
                tableRow.id = uuid + "-" + rowIndex;
                return tableRow;
            }, {});
        });
    }, [rows, beeTableColumns, deepFlattenObjectRow, uuid]);
    const getColumnKey = useCallback((column) => {
        var _a;
        return (_a = column.originalId) !== null && _a !== void 0 ? _a : column.id;
    }, []);
    const getRowKey = useCallback((row) => {
        return row.original.id;
    }, []);
    return (_jsx(StandaloneBeeTable, { scrollableParentRef: scrollableParentRef, getColumnKey: getColumnKey, getRowKey: getRowKey, tableId: id, isEditableHeader: false, headerLevelCountForAppendingRowIndexColumn: 1, headerVisibility: BeeTableHeaderVisibility.AllLevels, operationConfig: beeTableOperationConfig, columns: beeTableColumns, rows: beeTableRows, isReadOnly: true, enableKeyboardNavigation: true, shouldRenderRowIndexColumn: false, shouldShowRowsInlineControls: false, shouldShowColumnsInlineControls: false, resizerStopBehavior: ResizerStopBehavior.SET_WIDTH_ALWAYS }, void 0));
}
//# sourceMappingURL=DmnRunnerOutputsTable.js.map